Module excorpbe

 Use GlobalConstants

  Implicit None

! Parameters for the Perdew-Wang (PRB 45,13244 (1992)) LDA correlation      
  Real*8, private, parameter :: AA=0.0310907d0
  Real*8, private, parameter :: a1=0.21370d0
  Real*8, private, parameter :: b1=7.59570d0
  Real*8, private, parameter :: b2=3.58760d0
  Real*8, private, parameter :: b3=1.63820d0
  Real*8, private, parameter :: b4=0.49294d0

! Parameters from PBE paper (and in program from web)
  Real*8, private, parameter :: kappa= 0.804d0
  Real*8, private, parameter :: mu = 0.2195149727645171d0
  Real*8, private, parameter :: beta = 0.06672455060314922d0
  Real*8, private, parameter :: gamm = 0.03109069086965489503494086371273d0
  Real*8, private, parameter :: betabygamm=beta/gamm


  Contains

!**********************************************************
! Subroutine to calculate the LDA exchange correlation functionals
!   using the form of Perdew and Wang (PRB 45, 13244 (1992)
!   assuming no spin polarization
!  Inside this routine, energies are in Hartree units

 Subroutine pwldafunc(den,exc,vxc)
  Real*8, intent(IN) :: den    !density
  Real*8, intent(OUT) :: exc,vxc

! Variables depending on den
  Real*8 :: n,g,kf,rs,ks

  Real*8 :: ex,ec,pprs,decdrs
  Real*8 :: term

  n=den
  If (n < machine_zero)  then
    exc=0.d0; vxc=0.d0
    return
  EndIf

  kf=(3.d0*(pi**2)*n)**0.3333333333333333333333333333d0
  rs=(3.d0/(4.d0*pi*n))**0.3333333333333333333333333333d0
  ks=sqrt(4.d0*kf/pi)

  ex=-3.d0*kf/(4.d0*pi)
  pprs=sqrt(rs)*(b1+b3*rs)+rs*(b2+b4*rs)
  !ec=-2.d0*AA*(1.d0+a1*rs)*ddlog(1.d0+1.d0/(2.d0*AA*pprs))
  term=Logofterm(1.d0/(2.d0*AA*pprs))
  ec=-2.d0*AA*(1.d0+a1*rs)*term

  exc=ex+ec

  !decdrs=-(2.d0*AA*a1)*ddlog(1.d0+1.d0/(2*AA*pprs)) &
  decdrs=-(2.d0*AA*a1)*term &
      +((1.d0+a1*rs)*((b1+3*b3*rs)/(2.d0*sqrt(rs))+b2+2*b4*rs))/(pprs*(pprs+1.d0/(2.d0*AA)))

  vxc = (4.d0/3.d0)*ex+ec-(decdrs*rs)/3.d0

  if ((abs(exc).gt.1.d65).or.(abs(vxc).gt.1.d65)) then
      write(6,*) 'Problem in PW',n,rs,ec
  endif

  exc=2*exc; vxc=2*vxc      ! change to Rydberg units

  return
  end subroutine
!**********************************************************
! Subroutine to calculate the exchange correlation functionals
!   using the form of Perdew, Burke, and Ernzerhof (PRL 77, 3865 (1996))
!   assuming no spin polarization
!  Inside this routine, energies are in Hartree units

 Subroutine pbefunc(den,grad,fxc,dfxcdn,dfxcdgbg)
  Real*8, intent(IN) :: den,grad    !density, magnitude of grad(density)
  Real*8, intent(OUT) :: fxc,dfxcdn,dfxcdgbg

! Variables depending on den & grad
  Real*8 :: n,g,kf,rs,ks,s,t

  Real*8 :: ex,ec,Fx,H,A,pprs,ppt,At2,dFds,dHdt,decdrs,dHdrs,dHdA,dAdrs
  Real*8 :: term,dHdtbg,dFdsbg

  n=den
  If (n < machine_zero)  then
    fxc=0.d0; dfxcdn=0.d0; dfxcdgbg=0.d0
    return
  EndIf
  g=grad
  If (g < machine_zero) g=machine_zero

  kf=(3.d0*(pi**2)*n)**0.3333333333333333333333333333d0
  rs=(3.d0/(4.d0*pi*n))**0.3333333333333333333333333333d0
  ks=sqrt(4.d0*kf/pi)
  s=g/(2.d0*kf*n)
  t=g/(2.d0*ks*n)

  ex=-3.d0*kf/(4.d0*pi)
  pprs=sqrt(rs)*(b1+b3*rs)+rs*(b2+b4*rs)
  !ec=-2.d0*AA*(1.d0+a1*rs)*ddlog(1.d0+1.d0/(2.d0*AA*pprs))
  term=Logofterm(1.d0/(2.d0*AA*pprs))
  ec=-2.d0*AA*(1.d0+a1*rs)*term
  Fx=1.d0+kappa -kappa/(1.d0+(mu/kappa)*s*s)
  A=Aofec(ec)
  At2=A*t*t
  ppt=(1.d0+At2*(1.d0+At2))
  !H=gamm*ddlog(1.d0+(betabygamm)*(t*t)*((1.d0+At2)/ppt))
  H=gamm*Logofterm((betabygamm)*(t*t)*((1.d0+At2)/ppt))

  fxc=n*(ex*Fx+ec+H)

  dFds = (2.d0*mu*s)/(1.d0+(mu/kappa)*(s**2))**2
  dFdsbg = ((2.d0*mu)/(1.d0+(mu/kappa)*(s**2))**2)/(2.d0*kf*n)
  dHdt = (2.d0*t*beta*gamm*(1.d0+2.d0*At2))/((gamm*ppt+beta*t*t*(1.d0+At2))*ppt)
  dHdtbg = ((2.d0*beta*gamm*(1.d0+ & 
     2.d0*At2))/((gamm*ppt+beta*t*t*(1.d0+At2))*ppt))/(2.d0*ks*n)
  !decdrs=-(2.d0*AA*a1)*ddlog(1.d0+1.d0/(2*AA*pprs)) &
  decdrs=-(2.d0*AA*a1)*term &
      +((1.d0+a1*rs)*((b1+3*b3*rs)/(2.d0*sqrt(rs))+b2+2*b4*rs))/(pprs*(pprs+1.d0/(2.d0*AA)))
  dHdA=((2.d0+At2)*(At2*t*t*t*t*beta*gamm))/((gamm*ppt+beta*t*t*(1.d0+At2))*ppt)
  dAdrs=-ddexp(-ec/gamm)*A*A*decdrs/beta
  dHdrs=dHdA*dAdrs

  dfxcdn = (4.d0/3.d0)*ex*(Fx-dFds*s)+ec-(decdrs*rs)/3.d0+H-(dHdrs*rs)/3.d0 &
          - (7.d0/6.d0)*dHdt*t

  dfxcdgbg = ex*dFdsbg/(2.d0*kf) + dHdtbg/(2.d0*ks)

  if ((abs(fxc).gt.1.d65).or.(abs(dfxcdn).gt.1.d65).or.(abs(dfxcdgbg).gt.1.d65)) then
      write(6,*) 'Problem in PBE',n,g,rs,s,t,ec,A,H
  endif


  return
  end subroutine

!*******************************************************************
!
! Function Logofterm -- needed to take care of behavior for small term 
!  Evaluates log(1+term)

 Function Logofterm(term)

  Real*8 :: term, Logofterm

   If (abs(term)>machine_precision) then
     Logofterm=ddlog(1.d0+term)
   Else
     Logofterm=term
   endIf

 Return
 End Function

!*******************************************************************
!
! Function Aofec -- needed to take care of behavior for small ec

 Function Aofec(ec)

  Real*8 :: ec, Aofec

   If (abs(ec)>machine_precision) then
     Aofec=betabygamm/(ddexp(-ec/gamm)-1.d0)
   ElseIf (abs(ec)>machine_zero) then
     Aofec=beta/(-ec)
   Else
     Aofec=-beta*DSIGN(machine_infinity,ec)
   endIf

 Return
 End Function

!********************************************************************
!
! Subroutine radialexcpbe
!   Density(:) input on a uniform radial mesh of Npts
!   R(:) input mesh points
!   Exc - output integrated exchange correlation energy   -- in Rydberg units
!   vxc(:) -- output exchange correlation potential   -- iin Rydberg units
!
!********************************************************************

 Subroutine radialexcpbe(Npts,r,density,Exc,vxc)

  Implicit None

  Integer, intent(IN) ::Npts
  Real*8, intent(IN) :: r(Npts),density(Npts)
  Real*8, intent(OUT) :: Exc, vxc(Npts)

  Integer :: i,j 
  Real*8 :: gradient(Npts),gradmag(Npts),gxc(Npts),dgxcdr(Npts)
  Real*8 :: fxc(Npts),dfxcdn,dfxcdgbg,sgn,overint
  Real*8 :: h

  h=r(2)-r(1)
  Call nderiv(h,density,gradient,Npts)

  gradmag=abs(gradient)

  do i=1,Npts

    Call  pbefunc(density(i),gradmag(i),fxc(i),dfxcdn,dfxcdgbg)
    vxc(i)=dfxcdn
    gxc(i)=dfxcdgbg*gradient(i)

  enddo

  Call nderiv(h,gxc,dgxcdr,Npts)

  Do i=1,Npts
  
     fxc(i)=2*fxc(i)*4*pi*(r(i)*r(i))    !2* changes from Hartree to Rydberg
     vxc(i)=2*vxc(i)-2*dgxcdr(i)         !  units

  Enddo

  Exc = overint(Npts-1,h,fxc(2))

  return
  end subroutine

End Module
